@@ -8,26 +8,18 @@ from typing import (Iterable,
                     Union as Union_)
 
 from reprit.base import generate_repr
-from robust.linear import (SegmentsRelationship,
-                           segments_intersection,
-                           segments_relationship)
 
 from clipping.hints import (Mix,
                             Multipoint,
                             Multisegment,
-                            Point,
                             Segment)
 from . import bounding_box
-from .event import (BinaryEvent,
-                    NaryEvent)
-from .events_queue import (BinaryEventsQueue,
-                           BinaryEventsQueueKey,
-                           NaryEventsQueue,
-                           NaryEventsQueueKey)
+from .event import BinaryEvent
+from .events_queue import (LinearBinaryEventsQueue as BinaryEventsQueue,
+                           NaryEventsQueue)
 from .sweep_line import (BinarySweepLine,
                          NarySweepLine)
 from .utils import (all_equal,
-                    sort_pair,
                     to_multisegment_base,
                     to_multisegment_x_max,
                     to_rational_multisegment)
@@ -37,17 +29,10 @@ def merge_segments(segments: List[Segment],
                    accurate: bool) -> Iterable[Segment]:
     if not segments:
         return
-    events_queue = NaryEventsQueue()
     if accurate and not issubclass(to_multisegment_base(segments), Rational):
         segments = to_rational_multisegment(segments)
-    for start, end in segments:
-        if start > end:
-            start, end = end, start
-        start_event = NaryEvent(False, start, None)
-        end_event = NaryEvent(True, end, start_event)
-        start_event.complement = end_event
-        events_queue.push(start_event)
-        events_queue.push(end_event)
+    events_queue = NaryEventsQueue()
+    events_queue.register_segments(segments)
     sweep_line = NarySweepLine()
     while events_queue:
         event = events_queue.pop()
@@ -58,73 +43,16 @@ def merge_segments(segments: List[Segment],
                                             sweep_line.below(event))
                 sweep_line.remove(event)
                 if above_event is not None and below_event is not None:
-                    detect_intersection(events_queue, below_event, above_event)
+                    events_queue.detect_intersection(below_event, above_event)
                 yield event.segment
         elif event not in sweep_line:
             sweep_line.add(event)
             above_event, below_event = (sweep_line.above(event),
                                         sweep_line.below(event))
             if above_event is not None:
-                detect_intersection(events_queue, event, above_event)
+                events_queue.detect_intersection(event, above_event)
             if below_event is not None:
-                detect_intersection(events_queue, below_event, event)
-
-
-def detect_intersection(events_queue: NaryEventsQueue,
-                        below_event: NaryEvent,
-                        event: NaryEvent) -> None:
-    below_segment, segment = below_event.segment, event.segment
-    relationship = segments_relationship(below_segment, segment)
-    if relationship is SegmentsRelationship.OVERLAP:
-        # segments overlap
-        starts_equal = below_event.start == event.start
-        if starts_equal:
-            start_min = start_max = None
-        elif NaryEventsQueueKey(event) < NaryEventsQueueKey(below_event):
-            start_min, start_max = event, below_event
-        else:
-            start_min, start_max = below_event, event
-        ends_equal = event.end == below_event.end
-        if ends_equal:
-            end_min = end_max = None
-        elif (NaryEventsQueueKey(event.complement)
-              < NaryEventsQueueKey(below_event.complement)):
-            end_min, end_max = event.complement, below_event.complement
-        else:
-            end_min, end_max = below_event.complement, event.complement
-        if starts_equal:
-            # both line segments are equal or share the left endpoint
-            if not ends_equal:
-                divide_segment(events_queue, end_max.complement, end_min.start)
-        elif ends_equal:
-            # the line segments share the right endpoint
-            divide_segment(events_queue, start_min, start_max.start)
-        elif start_min is end_max.complement:
-            # one line segment includes the other one
-            divide_segment(events_queue, start_min, end_min.start)
-            divide_segment(events_queue, start_min, start_max.start)
-        else:
-            # no line segment includes the other one
-            divide_segment(events_queue, start_max, end_min.start)
-            divide_segment(events_queue, start_min, start_max.start)
-    elif (relationship is not SegmentsRelationship.NONE
-          and event.start != below_event.start
-          and event.end != below_event.end):
-        # segments do not intersect_multipolygons at endpoints
-        point = segments_intersection(below_segment, segment)
-        if point != below_event.start and point != below_event.end:
-            divide_segment(events_queue, below_event, point)
-        if point != event.start and point != event.end:
-            divide_segment(events_queue, event, point)
-
-
-def divide_segment(events_queue: NaryEventsQueue, event: NaryEvent,
-                   point: Point) -> None:
-    left_event = NaryEvent(False, point, event.complement)
-    right_event = NaryEvent(True, point, event)
-    event.complement.complement, event.complement = left_event, right_event
-    events_queue.push(left_event)
-    events_queue.push(right_event)
+                events_queue.detect_intersection(below_event, event)
 
 
 class Operation(ABC):
@@ -156,72 +84,9 @@ class Operation(ABC):
         Computes result of the operation.
         """
 
-    def detect_intersection(self,
-                            below_event: BinaryEvent,
-                            event: BinaryEvent) -> None:
-        below_segment, segment = below_event.segment, event.segment
-        relationship = segments_relationship(below_segment, segment)
-        if relationship is SegmentsRelationship.OVERLAP:
-            # segments overlap
-            if below_event.from_left is event.from_left:
-                raise ValueError('Segments of the same multisegment '
-                                 'should not overlap.')
-            starts_equal = below_event.start == event.start
-            if starts_equal:
-                start_min = start_max = None
-            elif (BinaryEventsQueueKey(event)
-                  < BinaryEventsQueueKey(below_event)):
-                start_min, start_max = event, below_event
-            else:
-                start_min, start_max = below_event, event
-
-            ends_equal = event.end == below_event.end
-            if ends_equal:
-                end_min = end_max = None
-            elif (BinaryEventsQueueKey(event.complement)
-                  < BinaryEventsQueueKey(below_event.complement)):
-                end_min, end_max = event.complement, below_event.complement
-            else:
-                end_min, end_max = below_event.complement, event.complement
-
-            if starts_equal:
-                # both line segments are equal or share the left endpoint
-                if not ends_equal:
-                    self.divide_segment(end_max.complement, end_min.start)
-            elif ends_equal:
-                # the line segments share the right endpoint
-                self.divide_segment(start_min, start_max.start)
-            elif start_min is end_max.complement:
-                # one line segment includes the other one
-                self.divide_segment(start_min, end_min.start)
-                self.divide_segment(start_min, start_max.start)
-            else:
-                # no line segment includes the other one
-                self.divide_segment(start_max, end_min.start)
-                self.divide_segment(start_min, start_max.start)
-        elif (relationship is not SegmentsRelationship.NONE
-              and event.start != below_event.start
-              and event.end != below_event.end):
-            # segments do not intersect_multipolygons at endpoints
-            point = segments_intersection(below_segment, segment)
-            if point != below_event.start and point != below_event.end:
-                self.divide_segment(below_event, point)
-            if point != event.start and point != event.end:
-                self.divide_segment(event, point)
-
-    def divide_segment(self, event: BinaryEvent, point: Point) -> None:
-        left_event = BinaryEvent(False, point, event.complement,
-                                 event.from_left)
-        right_event = BinaryEvent(True, point, event, event.from_left)
-        event.complement.complement, event.complement = left_event, right_event
-        self._events_queue.push(left_event)
-        self._events_queue.push(right_event)
-
     def fill_queue(self) -> None:
-        for segment in self.left:
-            self.register_segment(segment, True)
-        for segment in self.right:
-            self.register_segment(segment, False)
+        self._events_queue.register_segments(self.left, True)
+        self._events_queue.register_segments(self.right, False)
 
     def normalize_operands(self) -> None:
         left, right = self.left, self.right
@@ -240,30 +105,24 @@ class Operation(ABC):
                                             sweep_line.below(event))
                 sweep_line.remove(event)
                 if above_event is not None and below_event is not None:
-                    self.detect_intersection(below_event, above_event)
+                    self._events_queue.detect_intersection(below_event,
+                                                           above_event)
         else:
             sweep_line.add(event)
             above_event, below_event = (sweep_line.above(event),
                                         sweep_line.below(event))
             if above_event is not None:
-                self.detect_intersection(event, above_event)
+                self._events_queue.detect_intersection(event, above_event)
             if below_event is not None:
-                self.detect_intersection(below_event, event)
-
-    def register_segment(self, segment: Segment, from_left: bool) -> None:
-        start, end = sort_pair(segment)
-        start_event = BinaryEvent(False, start, None, from_left)
-        end_event = BinaryEvent(True, end, start_event, from_left)
-        start_event.complement = end_event
-        self._events_queue.push(start_event)
-        self._events_queue.push(end_event)
+                self._events_queue.detect_intersection(below_event, event)
 
     def sweep(self) -> List[BinaryEvent]:
         self.fill_queue()
         result = []
+        events_queue = self._events_queue
         sweep_line = BinarySweepLine()
-        while self._events_queue:
-            event = self._events_queue.pop()
+        while events_queue:
+            event = events_queue.pop()
             self.process_event(event, sweep_line)
             if event.is_right_endpoint:
                 result.append(event.complement)
@@ -293,10 +152,11 @@ class Difference(Operation):
     def sweep(self) -> List[BinaryEvent]:
         self.fill_queue()
         result = []
+        events_queue = self._events_queue
         sweep_line = BinarySweepLine()
         left_x_max = to_multisegment_x_max(self.left)
-        while self._events_queue:
-            event = self._events_queue.pop()
+        while events_queue:
+            event = events_queue.pop()
             start_x, _ = event.start
             if start_x > left_x_max:
                 break
@@ -331,11 +191,12 @@ class Intersection(Operation):
     def sweep(self) -> List[BinaryEvent]:
         self.fill_queue()
         result = []
+        events_queue = self._events_queue
         sweep_line = BinarySweepLine()
         min_max_x = min(to_multisegment_x_max(self.left),
                         to_multisegment_x_max(self.right))
-        while self._events_queue:
-            event = self._events_queue.pop()
+        while events_queue:
+            event = events_queue.pop()
             start_x, _ = event.start
             if start_x > min_max_x:
                 break
@@ -384,11 +245,12 @@ class CompleteIntersection(Operation):
     def sweep(self) -> List[BinaryEvent]:
         self.fill_queue()
         result = []
+        events_queue = self._events_queue
         sweep_line = BinarySweepLine()
         min_max_x = min(to_multisegment_x_max(self.left),
                         to_multisegment_x_max(self.right))
-        while self._events_queue:
-            event = self._events_queue.pop()
+        while events_queue:
+            event = events_queue.pop()
             start_x, _ = event.start
             if start_x > min_max_x:
                 break
